#include <SPI.h>
#include <Wire.h>
#include <Adafruit_GFX.h>       //Download: https://www.electronoobs.com/eng_arduino_Adafruit_GFX.php
#include <Adafruit_SSD1306.h>   //Downlaod: https://www.electronoobs.com/eng_arduino_Adafruit_SSD1306.php
Adafruit_SSD1306 display(4);    //Create screen variable


//Define all the Input/output pins of the board
int D2 = 2;     //MUX 1
int D3 = 3;     //MUX 2
int D4 = 4;     //Resistance 20K
int D5 = 5;     //Resistance 470K
int D6 = 6;     //Inductance pulse
int D7 = 7;     //Voltage divider R
int D8 = 8;     //Resistance 2K
int D9 = 9;     //Inductance out
int D10 = 10;   //Push button
int D11 = 11;   //Capacitance 10K
int D12 = 12;   //Voltage divider L
int D13 = 13;   //Capacitance 220K
int a0 = A0;    //Capacitance Read
int a1 = A1;    //Resistance diode 
int a2 = A2;    //Capacitance RC
int a3 = A3;    //MUX 0
int a6 = A6;    
int a7 = A7;


//Capacitance mode variables
unsigned long startTime;
unsigned long elapsedTime;
float microFarads;                
float nanoFarads;
#define resistorValue  7200.00F  //Remember, we've used a 10K resistor to charge the capacitor
bool cap_scale = true;
//Small scale
const float IN_STRAY_CAP_TO_GND = 56.88;
const float IN_CAP_TO_GND  = IN_STRAY_CAP_TO_GND;
const float R_PULLUP = 34.8;  
const int MAX_ADC_VALUE = 1023;



void setup() {
  Serial.begin(9600);
  //Define all pins as inputs but not the pins that we will use
  pinMode(a0, INPUT);
  pinMode(a1, INPUT);
  pinMode(a2, INPUT);
  pinMode(a3, INPUT);
  pinMode(a6, INPUT);
  pinMode(a7, INPUT);  
  pinMode(D2, INPUT);
  pinMode(D3, INPUT);
  pinMode(D4, INPUT);
  pinMode(D5, INPUT);
  pinMode(D6, INPUT);
  pinMode(D7, INPUT);
  pinMode(D8, INPUT);
  pinMode(D9, INPUT);
  pinMode(D10, INPUT);
  pinMode(D11, INPUT);
  pinMode(D12, INPUT);
  pinMode(D13, INPUT);

  display.begin(SSD1306_SWITCHCAPVCC, 0x3C);  //Start i2c communication
  display.setRotation(2);                     //Rotate screen 180 degree
  display.clearDisplay();                     //Clear the display data
  display.display();                          //Send data to the display
  delay(10);
}

void loop() {
  //MUX setup
  pinMode(a3, OUTPUT);  //S0 of mux
  pinMode(D2, OUTPUT);  //S1 of mux
  pinMode(D3, OUTPUT);  //S2 of mux
  //We set "001" so we activate MUX2 for Resistance mode (RC)
  digitalWrite(a3, HIGH);   //S0 to 1
  digitalWrite(D2, LOW);    //S1 to 0
  digitalWrite(D3, LOW);    //S2 to 0
  
  //This is the scale for 1uF to max value
  if(!cap_scale)
  {       
    pinMode(a0,INPUT);          //Capacitance input 1
    pinMode(a2, OUTPUT);        //Capacitance input 2
    pinMode(D11, OUTPUT);       //Charging pin
    delay(10);
    digitalWrite(a2, LOW); 
    digitalWrite(D11, HIGH);  
    startTime = micros();
    while(analogRead(a0) < 648){}    //end while
    
    elapsedTime= micros() - startTime;
    microFarads = ((float)elapsedTime / resistorValue);
      
    if (microFarads > 1)
    {
      Serial.print(microFarads);
      Serial.println(" uF");  
      delay(100);    
      display.clearDisplay();  
      display.setTextSize(2);
      display.setTextColor(WHITE);    
      display.setCursor(10,0);
      display.print("CAPACITOR");
      
      display.setTextSize(2);
      display.setTextColor(WHITE);    
      display.setCursor(0,18);
      display.print(microFarads);
      display.println(" uF");
      display.display();
    }
    
    else
    {
      /*nanoFarads = microFarads * 1000.0; 
      Serial.print(nanoFarads);
      Serial.println(" nF");  
      delay(500);  */
      cap_scale = true;       //We change the scale to MIN - 1uF
    }
    
    digitalWrite(D11, LOW);            
    pinMode(D13, OUTPUT);            
    digitalWrite(D13, LOW);        //discharging the capacitor
         
    while(analogRead(A0) > 0){}   //This while waits till the capaccitor is discharged    
    pinMode(D13, INPUT);           //this sets the pin to high impedance      
    //Serial.println("Discharging"); 
  }//end of upper scale


  if(cap_scale)
  {      
    pinMode(D13,INPUT);
    pinMode(D11,INPUT);
    pinMode(a2,OUTPUT);
    pinMode(a0, INPUT);
    
    digitalWrite(a2, HIGH);
    int val = analogRead(a0);
    digitalWrite(a2, LOW);

    if (val < 1000)
    {
      pinMode(a0, OUTPUT);  
      float capacitance = (float)val * IN_CAP_TO_GND / (float)(MAX_ADC_VALUE - val);
      
      Serial.print(capacitance,0);
      Serial.println(" pF");
      display.clearDisplay();  
      display.setTextSize(2);
      display.setTextColor(WHITE);    
      display.setCursor(10,0);
      display.print("CAPACITOR");
      
      display.setTextSize(2);
      display.setTextColor(WHITE);    
      display.setCursor(0,18);
      display.print(capacitance);
      display.println(" pF");
      display.display();
      delay(400);
    }
    
    else
    {
      pinMode(a0, OUTPUT);
      delay(1);
      pinMode(a2, INPUT_PULLUP);
      unsigned long u1 = micros();
      unsigned long t;
      int digVal;  
      do
      {
        digVal = digitalRead(a2);
        unsigned long u2 = micros();
        t = u2 > u1 ? u2 - u1 : u1 - u2;
      }         
      while ((digVal < 1) && (t < 400000L));

      pinMode(a2, INPUT);  
      val = analogRead(a2);
      digitalWrite(a0, HIGH);
      int dischargeTime = (int)(t / 1000L) * 5;
      delay(dischargeTime);   
      pinMode(a2, OUTPUT);  
      digitalWrite(a2, LOW);
      digitalWrite(a0, LOW);
      float capacitance = -(float)t / R_PULLUP / log(1.0 - (float)val / (float)MAX_ADC_VALUE);
                              
      if (capacitance > 1000.0)
      {          
        /*Serial.print(capacitance / 1000.0, 2);
        Serial.println(" uF");*/
        cap_scale = false;          //We change the scale to 1uF - max        
      }          
      else
      {          
        Serial.print(capacitance);
        Serial.println(" nF"); 
        display.clearDisplay();  
        display.setTextSize(2);
        display.setTextColor(WHITE);    
        display.setCursor(10,0);
        display.print("CAPACITOR");
        
        display.setTextSize(2);
        display.setTextColor(WHITE);    
        display.setCursor(0,18);
        display.print(capacitance);
        display.println(" nF");
        display.display(); 
                
      }
    }
    while (micros() % 1000 != 0);
  }////end of lower scalee

}
